# -*- coding: utf-8 -*-

###########################################################
# Finder Tags Butler
#
# Synchronize Mac OS Finder tags between several machines
#
# Copyright 2020 Borja González Seoane
#
# Contact: garaje@glezseoane.es
###########################################################


"""Finder Tags Butler errors module."""


class CorruptedManifestFileError(Exception):
    def __init__(self, *args):
        if args:
            self.path = args[0]
        else:
            self.path = None

    def __str__(self):
        if self.path:
            return (
                f"Tha manifest file '{self.path}' is corrupted and can "
                f"not be used by this program."
            )
        else:
            return "'CorruptedManifestFileError' has been raised."



# -*- coding: utf-8 -*-

###########################################################
# Finder Tags Butler
#
# Synchronize Mac OS Finder tags between several machines
#
# Copyright 2020 Borja González Seoane
#
# Contact: garaje@glezseoane.es
###########################################################

"""Finder Tags Butler test suite: integration tests for logic layer."""

import os
import random
import shutil
import socket
import string
import tempfile
import unittest
from unittest import TestCase

import yaml

from finder_tags_butler.logic_layer import (
    _get_children_of_path,
    save_manifest,
    dump_manifest,
    _validate_mainifest_file,
)
from finder_tags_butler.logic_tags import (
    add_finder_tag_for_path,
    get_finder_tags_for_path,
    rm_all_finder_tags_for_path,
)
from finder_tags_butler.properties import MANIFEST_FILE_NAME


class IntegrationTestSuiteLogicLayer(TestCase):
    def test_save_and_dump_manifest(self):
        """Test the full process to save and dump a manifest."""
        with tempfile.TemporaryDirectory() as sample_node:
            # Generate random stuff
            _generate_random_folders_tree(sample_node)

            # Select some random files and folders
            children = _get_children_of_path(sample_node)
            tagged_children = children[: (len(children) // 3)]  # Children subset

            # Create a list with a random tag name to each tagged child
            tags = []
            for _ in tagged_children:
                tags.append(
                    "".join(random.choice(string.ascii_letters) for i in range(15))
                )  # Random tag name

            # Tag children
            additional_tag_1 = "Multiple of eleven"
            additional_tag_2 = "Multiple of 11"
            for i in range(0, len(tagged_children)):
                add_finder_tag_for_path(tagged_children[i], tags[i])
                if not i % 11:
                    add_finder_tag_for_path(tagged_children[i], additional_tag_1)
                    add_finder_tag_for_path(tagged_children[i], additional_tag_2)

            # Save the manifest
            manifest_path = os.path.join(sample_node, MANIFEST_FILE_NAME)
            save_manifest(sample_node, manifest_path)

            # Remove all the tags
            for child in tagged_children:
                rm_all_finder_tags_for_path(child)

            # Now dump the saved manifest to restore the previous state
            dump_manifest(manifest_path, sample_node)

            # Check that the machine name is correctly stored
            with open(manifest_path, "r") as infile:
                file_manifest = yaml.load(infile, Loader=yaml.FullLoader)
            self.assertEqual(file_manifest.machine, socket.gethostname())

            # Get current state and check against the previous one
            for i in range(0, len(tagged_children)):
                res_tags = get_finder_tags_for_path(tagged_children[i])
                self.assertTrue(tags[i] in res_tags)
                if not i % 11:
                    self.assertTrue(additional_tag_1 in res_tags)
                    self.assertTrue(additional_tag_2 in res_tags)
                    # Achieve an empty list
                    res_tags.remove(tags[i])
                    res_tags.remove(additional_tag_1)
                    res_tags.remove(additional_tag_2)
                    self.assertFalse(res_tags)
                else:
                    # Achieve an empty list
                    res_tags.remove(tags[i])
                    self.assertFalse(res_tags)

    def test_manifest_files_validation(self):
        with tempfile.TemporaryDirectory() as sample_node:
            # Generate random stuff
            _generate_random_folders_tree(sample_node)

            # Select some random files and folders
            children = _get_children_of_path(sample_node)
            tagged_children = children[: (len(children) // 3)]  # Children subset

            # Create a list with a random tag name to each tagged child
            tags = []
            for _ in tagged_children:
                tags.append(
                    "".join(random.choice(string.ascii_letters) for i in range(15))
                )  # Random tag name

            # Tag children
            additional_tag_1 = "Multiple of eleven"
            additional_tag_2 = "Multiple of 11"
            for i in range(0, len(tagged_children)):
                add_finder_tag_for_path(tagged_children[i], tags[i])
                if not i % 11:
                    add_finder_tag_for_path(tagged_children[i], additional_tag_1)
                    add_finder_tag_for_path(tagged_children[i], additional_tag_2)

            # Save the manifest
            manifest_path = os.path.join(sample_node, MANIFEST_FILE_NAME)
            save_manifest(sample_node, manifest_path)

            # Save several copies of the manifest
            manifest_path_copy1 = f"{manifest_path}.copy1"
            manifest_path_copy2 = f"{manifest_path}.copy2"
            manifest_path_copy3 = f"{manifest_path}.copy3"
            shutil.copy(manifest_path, manifest_path_copy1)
            shutil.copy(manifest_path, manifest_path_copy2)
            shutil.copy(manifest_path, manifest_path_copy3)

            # Check validity of initial manifest
            self.assertTrue(_validate_mainifest_file(manifest_path))

            # Insert random dirty text at the head of the file
            with open(manifest_path_copy1, "w") as f:
                f.seek(0)
                f.write("fñv´´zñvsfvlausvbsvs<dfsdagfasb<cygidsvysGCV<UDS")

            self.assertFalse(_validate_mainifest_file(manifest_path_copy1))

            # Insert incomplete manifest representation
            with open(manifest_path_copy2, "w") as f:
                yaml.dump({"machine": "computer"}, f)

            self.assertFalse(_validate_mainifest_file(manifest_path_copy2))

            # Change name of component
            with open(manifest_path_copy3, "r") as f:
                filedata = f.read()
            filedata = filedata.replace("content", "wine")
            with open(manifest_path_copy3, "w") as f:
                f.write(filedata)

            self.assertFalse(_validate_mainifest_file(manifest_path_copy3))


if __name__ == "__main__":
    unittest.main()


def _generate_random_folders_tree(root_path: str, depth: int = 101) -> None:
    """Help method that generates a big folder tree with random files and
    folders.

    Contents won't be erased, so its recommended to use temporary paths to run
    this method.

    :param root_path: The path where create a Git repository structure and mock
        content.
    :param depth: The depth of the generated sources trees. I.e. more
        volume of random stuff created.
    """
    # Random seed
    r = random.randint(4, depth)

    subdirs = []
    for i in range(0, r // 4):
        subdir = os.path.join(
            root_path, "".join(random.choice(string.ascii_letters) for i in range(35))
        )
        os.mkdir(subdir)
        subdirs.append(subdir)

    for i in range(0, r // 2):
        subdir = random.choice(subdirs)
        if i % 2:
            open(os.path.join(subdir, f"file{i}.txt"), "a").close()
        else:
            subsubdir = os.path.join(subdir, f"subdirectory{i}")
            os.mkdir(subsubdir)
            with open(os.path.join(subsubdir, f"BIG_FILE{i}"), "wb") as f:
                f.write(os.urandom((i + 10) * 1024))



# -*- coding: utf-8 -*-

###########################################################
# Finder Tags Butler
#
# Synchronize Mac OS Finder tags between several machines
#
# Copyright 2020 Borja González Seoane
#
# Contact: garaje@glezseoane.es
###########################################################

"""Finder Tags Butler core controller module."""

import sys

from finder_tags_butler.cli_layer import run_parser, print_error, print_ok
from finder_tags_butler.logic_layer import *
from finder_tags_butler.properties import MANIFEST_FILE_NAME


def main():
    # First, run the parser
    path, opt = run_parser().values()

    # Calculate paths
    if not os.path.isdir(path):
        order_error_printing_and_exit(NotADirectoryError(path))
    manifest_path = os.path.join(path, MANIFEST_FILE_NAME)

    # Interpret the option selected by the user
    if opt == "save_opt":
        save_manifest(path=path, manifest_path=manifest_path)
        # If the process finish well...
        order_ok_printing_and_exit(
            f"The manifest of '{os.path.basename(os.path.abspath(path))}'"
            f" has been saved. 💾"
        )
    else:
        # Check manifest existence
        if not os.path.isfile(manifest_path):
            order_error_printing_and_exit(FileNotFoundError(manifest_path))

        try:
            if opt == "dump_opt":
                tagging_errors = dump_manifest(
                    manifest_path=manifest_path, path=path, force_overwriting=None
                )
            elif opt == "soft_dump_opt":
                tagging_errors = dump_manifest(
                    manifest_path=manifest_path, path=path, force_overwriting=False
                )
            elif opt == "hard_dump_opt":
                tagging_errors = dump_manifest(
                    manifest_path=manifest_path, path=path, force_overwriting=True
                )
            else:  # If the parser is updated with this if-block, this won't occur
                raise NotImplementedError
        except CorruptedManifestFileError as e:
            order_error_printing_and_exit(e)

        # noinspection PyUnboundLocalVariable
        for tag_error in tagging_errors:
            order_error_printing_without_exit(tag_error)
        # If the process finish well...
        order_ok_printing_and_exit(
            f"The manifest of '{os.path.basename(os.path.abspath(path))}' "
            f"has been dumped. 🏷"
        )


def order_ok_printing_and_exit(msg_text: str) -> None:
    """:param msg_text: The message text to print."""
    print_ok(msg_text)
    sys.exit(0)


def order_error_printing_without_exit(error: Exception) -> None:
    """:param error: The error to print."""
    print_error(error)


def order_error_printing_and_exit(error: Exception) -> None:
    """:param error: The error to print."""
    print_error(error)
    sys.exit(1)



# -*- coding: utf-8 -*-

###########################################################
# Finder Tags Butler
#
# Synchronize Mac OS Finder tags between several machines
#
# Copyright 2020 Borja González Seoane
#
# Contact: garaje@glezseoane.es
###########################################################

"""Finder Tags Butler command line interface."""

import argparse

# Style constants
COLOR_BOLD = "\033[1m"
COLOR_GREEN = "\033[92m"
COLOR_RED = "\033[91m"
COLOR_YELLOW = "\033[93m"
COLOR_BLUE = "\033[94m"
COLOR_CIAN = "\033[96m"
COLOR_RST = "\033[0m"  # Restore

# Predefined messages
MSG_OK = COLOR_GREEN + "[OK]" + COLOR_RST
MSG_ERROR = COLOR_RED + "[ERROR]" + COLOR_RST
MSG_WARNING = COLOR_YELLOW + "[WARNING]" + COLOR_RST


def run_parser() -> dict:
    """Parse the user input.

    Supported options are:
        - 'save_opt'.
        - 'dump_opt'.
        - 'hard_dump_opt'.
        - 'soft_dump_opt'.

    :return: A dict '{"path": args.path, "option": opt}'.
    """
    parser = argparse.ArgumentParser(
        description="Finder Tags Butler",
        epilog="Synchronize Mac OS Finder tags between several machines.",
    )
    parser.add_argument(
        "path", metavar="PATH", type=str, nargs=1, help="The directory where work...",
    )
    options = parser.add_mutually_exclusive_group(required=True)
    options.add_argument(
        "-s",
        "--save",
        dest="save_opt",
        action="store_true",
        help="Saves the tags of the 'path' directory to a manifest.",
    )
    options.add_argument(
        "-d",
        "--dump",
        dest="dump_opt",
        action="store_true",
        help="Dumps the tags of the manifest of the 'path' "
        "directory. If the manifest provides from other machine, "
        "also removes all the tags of the 'path' "
        "directory and children that are not in the manifest.",
    )
    options.add_argument(
        "-sd",
        "--soft-dump",
        dest="soft_dump_opt",
        action="store_true",
        help="Dumps the tags of the manifest of the 'path' "
        "directory, but does not remove any tag present in the current "
        "working directory and children.",
    )
    options.add_argument(
        "-hd",
        "--hard-dump",
        dest="hard_dump_opt",
        action="store_true",
        help="Dumps the tags of the manifest of the 'path' "
        "directory, removing all the tags of the 'path' "
        "directory and children that are not in the manifest.",
    )

    # Parse
    args = parser.parse_args()

    # Catch the selected option
    if args.save_opt:
        opt = "save_opt"
    elif args.dump_opt:
        opt = "dump_opt"
    elif args.soft_dump_opt:
        opt = "soft_dump_opt"
    elif args.hard_dump_opt:
        opt = "hard_dump_opt"

    # Return the full user input order
    # noinspection PyUnboundLocalVariable
    return {"path": args.path[0], "option": opt}


def print_ok(msg_text: str) -> None:
    """Print the input error with the proper error formatting.

    :param msg_text: The message content text.
    """
    print(f"{MSG_OK}: {msg_text}")


def print_error(error: Exception) -> None:
    """Print the input error with the proper error formatting.

    :param error: The error to print.
    """
    print(f"{MSG_ERROR}: {error.__str__()}")



# -*- coding: utf-8 -*-

###########################################################
# Finder Tags Butler
#
# Synchronize Mac OS Finder tags between several machines
#
# Copyright 2020 Borja González Seoane
#
# Contact: garaje@glezseoane.es
###########################################################

"""Finder Tags Butler main entrypoint."""

from finder_tags_butler import controller_layer


def main():
    # Delegate to the controller
    controller_layer.main()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

###########################################################
# Finder Tags Butler
#
# Synchronize Mac OS Finder tags between several machines
#
# Copyright 2020 Borja González Seoane
#
# Contact: garaje@glezseoane.es
###########################################################

"""Properties file."""

MANIFEST_FILE_NAME = ".ftb.yaml"
MANIFEST_HEAD_COMMENT = (
    "# Finder Tags Butler manifest file\n"
    "#\n"
    "# This file is used to synchronize Mac OS Finder tags related\n"
    "# to this directory between several machines\n\n"
    "# DO NOT EDIT THIS FILE BY HAND. COULD BE CORRUPTED!\n"
    "##############################################################\n\n"
)



# -*- coding: utf-8 -*-

###########################################################
# Finder Tags Butler
#
# Synchronize Mac OS Finder tags between several machines
#
# Copyright 2020 Borja González Seoane
#
# Contact: garaje@glezseoane.es
###########################################################

from setuptools import setup


with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

version = "0.1.dev2"

setup(
    name="finder-tags-butler",
    version=version,
    packages=["finder_tags_butler"],
    entry_points={"console_scripts": ["ftbutler=finder_tags_butler.__main__:main",],},
    python_requires=">=3.8.5",
    install_requires=["pyyaml==5.3.1", "mac-tag==0.0.0"],
    data_files=[("", ["requirements.txt"]), ("", ["README.md"]), ("", ["LICENSE"]),],
    url="https://github.com/bglezseoane/finder-tags-butler",
    download_url=f"https://github.com/bglezseoane/finder-tags-butler/archive/{version}.tar.gz",
    license="LICENSE",
    author="Borja González Seoane",
    author_email="garaje@glezseoane.es",
    description="Synchronize Mac OS Finder tags between several machines",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "Development Status :: 1 - Planning",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS",
        "Topic :: Utilities",
        "Topic :: System :: Filesystems",
    ],
)



# -*- coding: utf-8 -*-

###########################################################
# Finder Tags Butler
#
# Synchronize Mac OS Finder tags between several machines
#
# Copyright 2020 Borja González Seoane
#
# Contact: garaje@glezseoane.es
###########################################################

"""Finder Tags Butler test suite: unit tests for tags logic"""

import tempfile
import unittest
from unittest import TestCase

from finder_tags_butler.logic_tags import (
    add_finder_tag_for_path,
    get_finder_tags_for_path,
    rm_all_finder_tags_for_path,
    rm_finder_tag_for_path,
)


class UnitTestSuiteLogicTags(TestCase):
    def test_set_and_get_tag_on_file(self):
        """Test get and set Finder tags.

        This version uses a file.
        """
        with tempfile.NamedTemporaryFile() as f:
            sample_file_path = f.name
            manual_tag = "5w&sfdbdb!$·!!Y&%"
            add_finder_tag_for_path(sample_file_path, manual_tag)
            res_tags = get_finder_tags_for_path(sample_file_path)[0]
            self.assertEqual(manual_tag, res_tags)

    def test_set_and_get_tag_on_dir(self):
        """Test get and set Finder tags.

        This version uses a directory.
        """
        with tempfile.TemporaryDirectory() as sample_folder:
            manual_tag = "Comprobación"
            add_finder_tag_for_path(sample_folder, manual_tag)
            res_tags = get_finder_tags_for_path(sample_folder)[0]
            self.assertEqual(manual_tag, res_tags)

    def test_set_and_get_tags_on_dir(self):
        """Test get and set Finder tags."""
        with tempfile.TemporaryDirectory() as sample_folder:
            manual_tag1 = "Sample tag 1"
            manual_tag2 = "Sample tag 2"
            manual_tag3 = "Sample tag 3"
            add_finder_tag_for_path(
                sample_folder, manual_tag1,
            )
            add_finder_tag_for_path(
                sample_folder, manual_tag2,
            )
            add_finder_tag_for_path(
                sample_folder, manual_tag3,
            )

            res_tags = get_finder_tags_for_path(sample_folder)

            self.assertTrue(manual_tag1 in res_tags)
            self.assertTrue(manual_tag2 in res_tags)
            self.assertTrue(manual_tag3 in res_tags)

    def test_remove_tag_on_dir(self):
        """Test remove a Finder tag."""
        with tempfile.TemporaryDirectory() as sample_folder:
            manual_tag1 = "Sample tag 1"
            manual_tag2 = "Sample tag 2"
            manual_tag3 = "Sample tag 3"
            add_finder_tag_for_path(
                sample_folder, manual_tag1,
            )
            add_finder_tag_for_path(
                sample_folder, manual_tag2,
            )
            add_finder_tag_for_path(
                sample_folder, manual_tag3,
            )

            # Remove
            rm_finder_tag_for_path(sample_folder, manual_tag1)

            res_tags = get_finder_tags_for_path(sample_folder)

            self.assertTrue(manual_tag2 in res_tags)
            self.assertTrue(manual_tag3 in res_tags)

    def test_remove_all_tags_on_dir(self):
        """Test remove all Finder tags."""
        with tempfile.TemporaryDirectory() as sample_folder:
            manual_tag1 = "Sample tag 1"
            manual_tag2 = "Sample tag 2"
            manual_tag3 = "Sample tag 3"
            add_finder_tag_for_path(
                sample_folder, manual_tag1,
            )
            add_finder_tag_for_path(
                sample_folder, manual_tag2,
            )
            add_finder_tag_for_path(
                sample_folder, manual_tag3,
            )

            # Remove
            rm_all_finder_tags_for_path(sample_folder)

            res_tags = get_finder_tags_for_path(sample_folder)

            self.assertEqual(res_tags, [])


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

###########################################################
# Finder Tags Butler
#
# Synchronize Mac OS Finder tags between several machines
#
# Copyright 2020 Borja González Seoane
#
# Contact: garaje@glezseoane.es
###########################################################

"""Business logic layer."""

import os
import platform
from typing import Union, List

import yaml

from finder_tags_butler import properties
from finder_tags_butler.errors import CorruptedManifestFileError
from finder_tags_butler.logic_tags import (
    get_finder_tags_for_path,
    add_finder_tag_for_path,
    rm_all_finder_tags_for_path,
)


class TagAssociation:
    """Abstraction of a manifest content entry."""

    def __init__(self, path: str, tags: Union[List[str], None]):
        self.path = path
        self.tags = tags

    def __lt__(self, other):
        return self.path.__lt__(other.path)


class Manifest:
    """Abstraction of a manifest."""

    def __init__(self, content: List[TagAssociation] = None):
        """To load a manifest file, let 'content' set to 'None'."""
        if content is None:
            content = []
        self.content = content
        self.machine = platform.node()

    def save(self, path: str) -> None:
        """Save the current manifest object to a 'path''s YAML file.

        Warning: the path should be checked before call this function.

        :param path: The param of the output file.
        """
        with open(path, "w") as outfile:
            outfile.writelines(properties.MANIFEST_HEAD_COMMENT)
            yaml.dump(self, outfile)

    def load(self, path: str) -> None:
        """Read a 'path''s YAML file to the current 'Manifest' object.

        Warning: the path should be checked before call this function.

        :param path: The param of the output file.
        """
        with open(path, "r") as infile:
            file_manifest = yaml.load(infile, Loader=yaml.FullLoader)
            self.content = file_manifest.content
            self.machine = file_manifest.machine


def save_manifest(path: str, manifest_path: str,) -> None:
    """Save a manifest of the given 'path' into the given 'manifest_path'.

    Warning: the paths should be checked before call this function.

    :param path: The path to explore.
    :param manifest_path: The path of the output manifest (it would be
        overriding).
    """
    # Assert the paths are correct and absolutely
    path = os.path.abspath(os.path.expanduser(path))
    manifest_path = os.path.abspath(os.path.expanduser(manifest_path))

    # Get all the child files and folders recursively
    children = _get_children_of_path(path)

    # Prepare the content entries after explore the child elements
    content = []
    for child in children:
        tags = get_finder_tags_for_path(child)
        if not tags == []:
            content.append(TagAssociation(child, tags))

    # Finally, create the manifest and write it to a file
    manifest = Manifest(content)

    # Save the manifest
    manifest.save(manifest_path)


def dump_manifest(
    manifest_path: str, path: str, force_overwriting: Union[bool, None] = False
) -> [Exception]:
    """Dump a 'manifest_path''s manifest writing tags into the node's 'path'
    location.

    By default, if the manifest provides from another machine, the tags present
    in the target node but not in the manifest will be removed. If the manifest
    provides from the same machine, these tags will be preserved and a warning
    will be presented. Using the 'force_overwriting' param this behaviour can be
    forced.

    Warning: the paths should be checked before call this function.

    :param manifest_path: The path of the input manifest.
    :param path: The path to apply the manifest.
    :param force_overwriting: Passing this param as 'True', the tags of the
        target node will be always totally erased before dumping the
        manifest. Passing as 'False', they won't be removed never.
        Letting as 'None', they will be removed if the manifest
        provides from other machine.
    :return: A list of errors of the tags that have not been correctly
        processed.
    :raise: CorruptedManifestFileError, if the manifest file is corrupted.
    """
    # Assert the paths are correct and absolutely
    manifest_path = os.path.abspath(os.path.expanduser(manifest_path))
    path = os.path.abspath(os.path.expanduser(path))

    # Read the manifest
    if not _validate_mainifest_file(manifest_path):  # First check integrity
        raise CorruptedManifestFileError(manifest_path)
    manifest = Manifest()
    manifest.load(manifest_path)

    # Get all the child files and folders recursively
    children = _get_children_of_path(path)

    # First clean all the tags of the node, if it apply
    if force_overwriting is True or (
        platform.node() != manifest.machine and force_overwriting is not False
    ):
        for child in children:
            rm_all_finder_tags_for_path(child)

    # Set new tags
    tagging_errors = []
    try:
        for child in manifest.content:
            for tag in child.tags:
                if os.path.exists(child.path):
                    add_finder_tag_for_path(child.path, tag)
                else:
                    tagging_errors.append(FileNotFoundError(child.path))
                    continue  # Do not raise exception, the full process must go on
    except AttributeError:
        raise CorruptedManifestFileError(manifest_path)

    return tagging_errors


def _get_children_of_path(path: str) -> [str]:
    """Return all the children files and folders recursively.

    Discard hidden files and folders.

    :param path: The path of the node directory to explore.
    :return: A list with the children files and folders paths.
    """
    children = [path]
    for root, directories, files in os.walk(path):
        # Disable hidden contents
        directories[:] = [d for d in directories if not d.startswith(".")]
        files[:] = [d for d in files if not d.startswith(".")]

        for directory in directories:
            children.append(os.path.join(root, directory))
        for file in files:
            children.append(os.path.join(root, file))

    return children


# noinspection PyStatementEffect
def _validate_mainifest_file(manifest_path: str) -> bool:
    """Check the integrity of a manifest file.

    :param manifest_path: The path of the manifest to validate
    :return: True or false of the manifest is valid or not, respectively.
    """
    try:
        manifest = Manifest()
        manifest.load(manifest_path)

        # Check access to all the stuff
        manifest.machine
        manifest.content
        for child in manifest.content:
            child.path
            child.tags
    except AttributeError:
        return False

    return True



# -*- coding: utf-8 -*-

###########################################################
# Finder Tags Butler
#
# Synchronize Mac OS Finder tags between several machines
#
# Copyright 2020 Borja González Seoane
#
# Contact: garaje@glezseoane.es
###########################################################

"""Tags functions.

This file simply closures some functions to manage Finder tags.
"""

import mac_tag


def get_finder_tags_for_path(path: str) -> [dict]:
    """Returns the Finder tags for a given file or folder.

    :param path: The path of the file of folder to examine.
    :raise FileNotFoundError: If the path does not points to anything reachable.
    :return: A list of tags as a dict with their titles and colors names.
    """
    try:
        tags_dict = mac_tag.get(path)
        tags = []
        """Access in a unnatural way due to some conflicts with complicated 
        paths... It is known that the dict will only contain one value"""
        for e in tags_dict:
            tags.extend(tags_dict[e])
        return tags
    except FileNotFoundError:
        raise FileNotFoundError(path)


def add_finder_tag_for_path(path: str, tag: str) -> None:
    """Set a Finder tag for a given file or folder.

    :param path: The path of the file of folder to edit.
    :param tag: The tag name to set.
    """
    if tag:
        mac_tag.add(tag, path)
    else:
        raise ValueError("Null tags are not valid")


def rm_finder_tag_for_path(path: str, tag: str) -> None:
    """Remove a Finder tag for a given file or folder.

    :param path: The path of the file of folder to edit.
    :param tag: The tag name to remove.
    """
    mac_tag.remove(tag, path)


def rm_all_finder_tags_for_path(path: str) -> None:
    """Remove all existent Finder tags for a given file or folder.

    :param path: The path of the file of folder to edit.
    """
    tags = get_finder_tags_for_path(path)
    for tag in tags:
        mac_tag.remove(tag, path)
